﻿#include "hmg.ch"
#include "PdfView.ch"

STATIC cPanel
STATIC lOpenAtOnce
STATIC nHFocusMenu

MEMVAR _HMG_SYSDATA


FUNCTION Main()

  cPanel := "SumatraPanel"
  lOpenAtOnce := .T.

  SET FONT TO "MS Shell Dlg", 8

  DEFINE WINDOW PdfView;
    ROW      0;
    COL      0;
    WIDTH  735;
    HEIGHT 575;
    TITLE  "PdfView";
    MAIN;
    ON SIZE     ResizeControls();
    ON MAXIMIZE ResizeControls();
    ON RELEASE  Sumatra_PanelClose(cPanel);
    ON INIT     (PdfView.Files.VALUE := 1)

    DEFINE GRID Files
      ROW              0
      COL              0
      WIDTH          150
      ITEMS          GetPdfFiles()
      HEADERS        {"Files"}
      WIDTHS         {150 - 4}
      CELLNAVIGATION .F.
      ONDBLCLICK     OpenPdfFile(PdfView.Files.CellEx(PdfView.Files.VALUE, 1))
      ONCHANGE       If(lOpenAtOnce, OpenPdfFile(PdfView.Files.CellEx(PdfView.Files.VALUE, 1)), NIL)
      ONKEY          GridOnKey()
    END GRID

    DEFINE STATUSBAR
      STATUSITEM ""
      STATUSITEM "" WIDTH 120
    END STATUSBAR

    DEFINE TIMER PdfViewTimer;
      INTERVAL 100;
      ACTION   SetStatusPage()

    DEFINE MAINMENU
      DEFINE POPUP "&File"
        MENUITEM "Open"                    NAME FileOpen       ACTION OpenPdfFile(PdfView.Files.CellEx(PdfView.Files.VALUE, 1))
        MENUITEM "Open at page"            NAME FileOpenAt     ACTION OpenPdfFile(PdfView.Files.CellEx(PdfView.Files.VALUE, 1), .T.)
        MENUITEM e"Close\tCtrl+W"          NAME FileClose      ACTION OpenPdfFile()
        SEPARATOR
        MENUITEM e"Save as...\tCtrl+S"     NAME FileSaveAs     ACTION Sumatra_FileSaveAs(cPanel)
        MENUITEM e"Print...\tCtrl+P"       NAME FilePrint      ACTION Sumatra_FilePrint(cPanel)
        MENUITEM e"Properties\tCtrl+D"     NAME FileProperties ACTION Sumatra_FileProperties(cPanel)
        SEPARATOR
        MENUITEM e"Exit\tEsc"              NAME AppExit        ACTION ThisWindow.RELEASE
      END POPUP
      DEFINE POPUP "&Page"
        MENUITEM e"Go to...\tCtrl+G"       NAME PageGoTo       ACTION Sumatra_PageGoTo(cPanel)
        SEPARATOR
        MENUITEM "Previous"                NAME PagePrev       ACTION Sumatra_PageGoTo(cPanel, -1)
        MENUITEM "Next"                    NAME PageNext       ACTION Sumatra_PageGoTo(cPanel,  1)
        MENUITEM "First"                   NAME PageFirst      ACTION Sumatra_PageGoTo(cPanel, -2)
        MENUITEM "Last"                    NAME PageLast       ACTION Sumatra_PageGoTo(cPanel,  2)
      END POPUP
      DEFINE POPUP "&Find"
        MENUITEM e"Text...\tCtrl+F"        NAME FindText       ACTION Sumatra_FindText(cPanel)
        SEPARATOR
        MENUITEM e"Previous\tShift+F3"     NAME FindPrev       ACTION Sumatra_FindText(cPanel, -1)
        MENUITEM e"Next\tF3"               NAME FindNext       ACTION Sumatra_FindText(cPanel,  1)
      END POPUP
      DEFINE POPUP "&Zoom"
        MENUITEM e"Size down\tCtrl+Minus"  NAME SizeDown       ACTION Sumatra_Zoom(cPanel, -1)
        MENUITEM e"Size up\tCtrl+Plus"     NAME SizeUp         ACTION Sumatra_Zoom(cPanel,  1)
        SEPARATOR
        MENUITEM e"Fit page\tCtrl+0"       NAME FitPage        ACTION Sumatra_Zoom(cPanel, 2)
        MENUITEM e"Real size\tCtrl+1"      NAME SizeReal       ACTION Sumatra_Zoom(cPanel, 3)
        MENUITEM e"Fit width\tCtrl+2"      NAME FitWidth       ACTION Sumatra_Zoom(cPanel, 4)
        SEPARATOR
        MENUITEM e"Zoom factor...\tCtrl+Y" NAME ZoomFactor     ACTION Sumatra_Zoom(cPanel)
      END POPUP
      DEFINE POPUP "&Rotate"
        MENUITEM e"Left\tCtrl+Shift+Minus" NAME RotateLeft     ACTION Sumatra_Rotate(cPanel, -1)
        MENUITEM e"Right\tCtrl+Shift+Plus" NAME RotateRight    ACTION Sumatra_Rotate(cPanel,  1)
        MENUITEM e"180°\tCtrl+Shift+Num*"  NAME Rotate180      ACTION Sumatra_Rotate(cPanel)
      END POPUP
      DEFINE POPUP "&Settings"
        MENUITEM e"Bookmarks\tF12"         NAME Bookmarks      ACTION Sumatra_Bookmarks(cPanel, ! Sumatra_Bookmarks(cPanel))
        MENUITEM e"Toolbar\tF8"            NAME Toolbar        ACTION Sumatra_Toolbar(cPanel, ! Sumatra_Toolbar(cPanel))
        SEPARATOR
        MENUITEM "Open pdf at once"        NAME OpenAtOnce     ACTION (lOpenAtOnce := ! lOpenAtOnce)
      END POPUP
    END MENU
  END WINDOW

  Sumatra_PanelCreate("PdfView", cPanel, 0, PdfView.Files.WIDTH)
  HMG_ChangeWindowStyle(GetFormHandle(cPanel), WS_SIZEBOX, NIL, .F., .T.)

  ResizeControls()
  SetOnKey(.T.)
  InstallEventHandler("EventHandler")

  PdfView.CENTER
  PdfView.ACTIVATE

RETURN NIL


FUNCTION EventHandler(nHWnd, nMsg, nWParam, nLParam)
  LOCAL nMaxY
  LOCAL aRect1, aRect2
  LOCAL nCol

  IF nHWnd == PdfView.HANDLE
    SWITCH nMsg
      CASE 36 /*WM_GETMINMAXINFO*/
        SetMinMaxTrackSize(nLParam, 400, 300)
        EXIT

      CASE 274 /*WM_SYSCOMMAND*/
        IF HB_BitAnd(nWParam, 0xFFF0) == 0xF100 /*SC_KEYMENU*/
          //for Win-10
          IF GetFocus() != PdfView.Files.HANDLE
            nHFocusMenu := GetFocus()
            PdfView.Files.SETFOCUS
            PostMessage(nHWnd, nMsg, nWParam, nLParam)
            RETURN 1
          ENDIF
        ENDIF
        RETURN 0

      CASE 278 /*WM_INITMENU*/
        EnableCommands()
        SetOnKey(.F.)
        RETURN 0

      CASE 530 /*WM_EXITMENULOOP*/
        SetOnKey(.T.)

        //for Win-10
        IF IsWindowEnabled(nHFocusMenu)
          SetFocus(nHFocusMenu)
          nHFocusMenu := 0
        ENDIF
        RETURN 0
    ENDSWITCH

  ELSEIF _IsWindowDefined(cPanel) .and. (nHWnd == GetFormHandle(cPanel))
    SWITCH nMsg
      CASE 36 /*WM_GETMINMAXINFO*/
        nMaxY := PdfView.CLIENTAREAHEIGHT - GetWindowHeight(PdfView.STATUSBAR.HANDLE)
        SetMinMaxTrackSize(nLParam, 200, nMaxY, PdfView.CLIENTAREAWIDTH - 100, nMaxY)
        EXIT

      CASE 532 /*WM_SIZING*/
        aRect1 := Array(4)
        aRect2 := Array(4)

        GetWindowRect(GetFormHandle(cPanel), aRect1)
        GetRectArray(nLParam, aRect2)

        SWITCH nWParam
          CASE 1 /*WMSZ_LEFT*/
          CASE 4 /*WMSZ_TOPLEFT*/
          CASE 7 /*WMSZ_BOTTOMLEFT*/
            IF aRect1[1] != aRect2[1]
              nCol := ScreenToClientCol(PdfView.HANDLE, aRect2[1])
              PdfView.Files.WIDTH := nCol
              PdfView.Files.ColumnWIDTH(1) := nCol - 4
              Sumatra_PanelSize(cPanel, NIL, NIL, PdfView.CLIENTAREAWIDTH - nCol + (GetProperty(cPanel, "WIDTH") - GetProperty(cPanel, "CLIENTAREAWIDTH")) / 2, NIL)
            ENDIF
            EXIT
          OTHERWISE
            SetRectArray(nLParam, aRect1)
        ENDSWITCH
        EXIT
    ENDSWITCH
  ENDIF

RETURN NIL


FUNCTION SetOnKey(lSet)

  IF lSet
    ON KEY TAB                    OF PdfView ACTION SetFocusNextDlgTab(.F.)
    ON KEY SHIFT+TAB              OF PdfView ACTION SetFocusNextDlgTab(.T.)
    ON KEY CONTROL+TAB            OF PdfView ACTION SetFocusBookmarks()
    ON KEY CONTROL+W              OF PdfView ACTION OpenPdfFile()
    ON KEY CONTROL+S              OF PdfView ACTION Sumatra_FileSaveAs(cPanel)
    ON KEY CONTROL+P              OF PdfView ACTION Sumatra_FilePrint(cPanel)
    ON KEY CONTROL+D              OF PdfView ACTION Sumatra_FileProperties(cPanel)
    ON KEY ESCAPE                 OF PdfView ACTION PdfView.RELEASE
    ON KEY CONTROL+G              OF PdfView ACTION Sumatra_PageGoTo(cPanel)
    ON KEY CONTROL+F              OF PdfView ACTION Sumatra_FindText(cPanel)
    ON KEY SHIFT+F3               OF PdfView ACTION Sumatra_FindText(cPanel, -1)
    ON KEY F3                     OF PdfView ACTION Sumatra_FindText(cPanel,  1)
    ON KEY CONTROL+MINUS          OF PdfView ACTION Sumatra_Zoom(cPanel, -1)
    ON KEY CONTROL+SUBTRACT       OF PdfView ACTION Sumatra_Zoom(cPanel, -1)
    ON KEY CONTROL+PLUS           OF PdfView ACTION Sumatra_Zoom(cPanel,  1)
    ON KEY CONTROL+ADD            OF PdfView ACTION Sumatra_Zoom(cPanel,  1)
    ON KEY CONTROL+0              OF PdfView ACTION Sumatra_Zoom(cPanel, 2)
    ON KEY CONTROL+NUM0           OF PdfView ACTION Sumatra_Zoom(cPanel, 2)
    ON KEY CONTROL+1              OF PdfView ACTION Sumatra_Zoom(cPanel, 3)
    ON KEY CONTROL+NUM1           OF PdfView ACTION Sumatra_Zoom(cPanel, 3)
    ON KEY CONTROL+2              OF PdfView ACTION Sumatra_Zoom(cPanel, 4)
    ON KEY CONTROL+NUM2           OF PdfView ACTION Sumatra_Zoom(cPanel, 4)
    ON KEY CONTROL+7              OF PdfView ACTION NIL
    ON KEY CONTROL+NUM7           OF PdfView ACTION NIL
    ON KEY CONTROL+8              OF PdfView ACTION NIL
    ON KEY CONTROL+NUM8           OF PdfView ACTION NIL
    ON KEY CONTROL+Y              OF PdfView ACTION Sumatra_Zoom(cPanel)
    ON KEY CONTROL+SHIFT+MINUS    OF PdfView ACTION Sumatra_Rotate(cPanel, -1)
    ON KEY CONTROL+SHIFT+SUBTRACT OF PdfView ACTION Sumatra_Rotate(cPanel, -1)
    ON KEY CONTROL+SHIFT+PLUS     OF PdfView ACTION Sumatra_Rotate(cPanel,  1)
    ON KEY CONTROL+SHIFT+ADD      OF PdfView ACTION Sumatra_Rotate(cPanel,  1)
    ON KEY CONTROL+SHIFT+MULTIPLY OF PdfView ACTION Sumatra_Rotate(cPanel)
    ON KEY CONTROL+SHIFT+DIVIDE   OF PdfView ACTION Sumatra_Rotate(cPanel)
    ON KEY F8                     OF PdfView ACTION Sumatra_Toolbar(cPanel, ! Sumatra_Toolbar(cPanel))
    ON KEY ALT+RETURN             OF PdfView ACTION If(IsMaximized(PdfView.HANDLE), PdfView.RESTORE, PdfView.MAXIMIZE)
  ELSE
    RELEASE KEY TAB                    OF PdfView
    RELEASE KEY SHIFT+TAB              OF PdfView
    RELEASE KEY CONTROL+TAB            OF PdfView
    RELEASE KEY CONTROL+W              OF PdfView
    RELEASE KEY CONTROL+S              OF PdfView
    RELEASE KEY CONTROL+P              OF PdfView
    RELEASE KEY CONTROL+D              OF PdfView
    RELEASE KEY ESCAPE                 OF PdfView
    RELEASE KEY CONTROL+G              OF PdfView
    RELEASE KEY CONTROL+F              OF PdfView
    RELEASE KEY SHIFT+F3               OF PdfView
    RELEASE KEY F3                     OF PdfView
    RELEASE KEY CONTROL+MINUS          OF PdfView
    RELEASE KEY CONTROL+SUBTRACT       OF PdfView
    RELEASE KEY CONTROL+PLUS           OF PdfView
    RELEASE KEY CONTROL+ADD            OF PdfView
    RELEASE KEY CONTROL+0              OF PdfView
    RELEASE KEY CONTROL+NUM0           OF PdfView
    RELEASE KEY CONTROL+1              OF PdfView
    RELEASE KEY CONTROL+NUM1           OF PdfView
    RELEASE KEY CONTROL+2              OF PdfView
    RELEASE KEY CONTROL+NUM2           OF PdfView
    RELEASE KEY CONTROL+7              OF PdfView
    RELEASE KEY CONTROL+NUM7           OF PdfView
    RELEASE KEY CONTROL+8              OF PdfView
    RELEASE KEY CONTROL+NUM8           OF PdfView
    RELEASE KEY CONTROL+Y              OF PdfView
    RELEASE KEY CONTROL+SHIFT+MINUS    OF PdfView
    RELEASE KEY CONTROL+SHIFT+SUBTRACT OF PdfView
    RELEASE KEY CONTROL+SHIFT+PLUS     OF PdfView
    RELEASE KEY CONTROL+SHIFT+ADD      OF PdfView
    RELEASE KEY CONTROL+SHIFT+MULTIPLY OF PdfView
    RELEASE KEY CONTROL+SHIFT+DIVIDE   OF PdfView
    RELEASE KEY F8                     OF PdfView
    RELEASE KEY ALT+RETURN             OF PdfView
  ENDIF

RETURN NIL


FUNCTION GridOnKey()

  IF HMG_GetLastVirtualKeyDown() == VK_F12
    HMG_CleanLastVirtualKeyDown()

    IF (GetKeyState(VK_CONTROL) >= 0) .and. (GetKeyState(VK_SHIFT) >= 0) .and. (GetKeyState(VK_MENU) >= 0)
      Sumatra_Bookmarks(cPanel, ! Sumatra_Bookmarks(cPanel))
      PdfView.Files.SETFOCUS
    ENDIF
  ENDIF

RETURN NIL


FUNCTION SetFocusNextDlgTab(lPrevious)

  SetFocus(GetNextDlgTabItem(PdfView.HANDLE, GetFocus(), lPrevious))

RETURN NIL


FUNCTION SetFocusBookmarks()
  LOCAL nHBook := Sumatra_BookmarksHandle(cPanel)

  IF (nHBook != 0) .and. IsWindowVisible(nHBook)
    IF GetFocus() == nHBook
      SetFocus(Sumatra_FrameHandle(cPanel))
    ELSE
      SetFocus(nHBook)
    ENDIF
  ENDIF

RETURN NIL


FUNCTION ResizeControls()
  LOCAL nMainCW := PdfView.CLIENTAREAWIDTH
  LOCAL nMainCH := PdfView.CLIENTAREAHEIGHT - GetWindowHeight(PdfView.STATUSBAR.HANDLE)
  LOCAL nFilesW := PdfView.Files.WIDTH
  LOCAL nFrameW := (GetProperty(cPanel, "WIDTH") - GetProperty(cPanel, "CLIENTAREAWIDTH")) / 2

  IF (nMainCW - nFilesW) < (200 - nFrameW)
    nFilesW := nMainCW - 200 - nFrameW
    PdfView.Files.WIDTH := nFilesW
    PdfView.Files.ColumnWIDTH(1) := nFilesW - 4
  ENDIF

  PdfView.Files.HEIGHT := nMainCH

  Sumatra_PanelSize(cPanel, NIL, nFilesW, nMainCW - nFilesW + nFrameW, nMainCH)

RETURN NIL


FUNCTION EnableCommands()
  LOCAL lPdfCount := (PdfView.Files.ITEMCOUNT > 0)
  LOCAL lOpened   := (Sumatra_FrameHandle(cPanel) != 0)
  LOCAL lLoaded   := (lOpened .and. (Sumatra_PageNumber(cPanel) > 0))

  PdfView.FileOpen.ENABLED       := lPdfCount
  PdfView.FileOpenAt.ENABLED     := lPdfCount
  PdfView.FileClose.ENABLED      := lOpened
  PdfView.FileSaveAs.ENABLED     := lLoaded
  PdfView.FilePrint.ENABLED      := lLoaded
  PdfView.FileProperties.ENABLED := lLoaded
  PdfView.PageGoto.ENABLED       := lLoaded
  PdfView.PagePrev.ENABLED       := lLoaded
  PdfView.PageNext.ENABLED       := lLoaded
  PdfView.PageFirst.ENABLED      := lLoaded
  PdfView.PageLast.ENABLED       := lLoaded
  PdfView.FindText.ENABLED       := lLoaded
  PdfView.FindPrev.ENABLED       := lLoaded
  PdfView.FindNext.ENABLED       := lLoaded
  PdfView.SizeDown.ENABLED       := lLoaded
  PdfView.SizeUp.ENABLED         := lLoaded
  PdfView.FitPage.ENABLED        := lLoaded
  PdfView.SizeReal.ENABLED       := lLoaded
  PdfView.FitWidth.ENABLED       := lLoaded
  PdfView.ZoomFactor.ENABLED     := lLoaded
  PdfView.Rotate180.ENABLED      := lLoaded
  PdfView.RotateLeft.ENABLED     := lLoaded
  PdfView.RotateRight.ENABLED    := lLoaded
  PdfView.Bookmarks.ENABLED      := Sumatra_BookmarksExist(cPanel)
  PdfView.Toolbar.ENABLED        := lOpened

  PdfView.Bookmarks.CHECKED  := Sumatra_Bookmarks(cPanel)
  PdfView.Toolbar.CHECKED    := Sumatra_Toolbar(cPanel)
  PdfView.OpenAtOnce.CHECKED := lOpenAtOnce

RETURN NIL


/*
  OpenPdfFile(cFile)        - open file
  OpenPdfFile(cFile, lPage) - open file at page
  OpenPdfFile()             - close file
*/
FUNCTION OpenPdfFile(cFile, lPage)
  STATIC lBookmarks := .T.
  STATIC lToolbar   := .F.
  LOCAL  nPage
  LOCAL  nHFocus
  LOCAL  nError
  LOCAL  cTitle

  IF PdfView.Files.VALUE == 0
    RETURN NIL
  ENDIF

  IF HB_IsLogical(lPage) .and. lPage
    nPage := Val(InputBox("Page:", "Open file at page", "1"))

    IF nPage < 1
      RETURN NIL
    ENDIF
  ENDIF

  nHFocus := GetFocus()

  IF Sumatra_BookmarksExist(cPanel)
    lBookmarks := Sumatra_Bookmarks(cPanel)
  ENDIF

  IF Sumatra_FrameHandle(cPanel) != 0
    lToolbar := Sumatra_Toolbar(cPanel)
  ENDIF

  nError := Sumatra_PanelCreate("PdfView", cPanel, NIL, NIL, NIL, NIL, cFile, nPage, lBookmarks, lToolbar)
  cTitle := Sumatra_FileName(cPanel)

  PdfView.TITLE := "PdfView" + If(Empty(cTitle), "", ": " + cTitle)
  PdfView.STATUSBAR.Item(1) := If(Empty(cTitle), "", GetCurrentFolder() + "\" + cTitle)

  IF nError == -1
    MsgBox("SumatraPDF executable file not found!")
  ELSEIF nError == -2
    MsgBox(cFile + e"\n\nThis file does not exist!")
  ELSE
    HMG_ChangeWindowStyle(GetFormHandle(cPanel), WS_SIZEBOX, NIL, .F., .T.)
    SetFocus(If((nHFocus == PdfView.Files.HANDLE) .or. (Sumatra_PageCount(cPanel) == 0), PdfView.Files.HANDLE, Sumatra_FrameHandle(cPanel)))
  ENDIF

RETURN NIL


FUNCTION GetPdfFiles()
  LOCAL aFiles := Directory("*.pdf")
  LOCAL nCount := Len(aFiles)
  LOCAL n

  FOR n := 1 TO nCount
   aSize(aFiles[n], 1)
  NEXT

RETURN aFiles


FUNCTION SetStatusPage()
  LOCAL nCount := Sumatra_PageCount(cPanel)
  LOCAL cText  := ""

  IF nCount > 0
    cText := "Page: " + HB_NtoS(Sumatra_PageNumber(cPanel)) + '/' + HB_NtoS(nCount)
  ENDIF

  PdfView.STATUSBAR.Item(2) := cText

RETURN NIL


#pragma BEGINDUMP

#include "SET_COMPILE_HMG_UNICODE.ch"
#include "HMG_UNICODE.h"

#include <windows.h>
#include "hbapi.h"


       //SetMinMaxTrackSize(lParam, nMinX, nMinY, nMaxX, nMaxY)
HB_FUNC( SETMINMAXTRACKSIZE )
{
  MINMAXINFO *MinMax = (MINMAXINFO *) HMG_parnl(1);

  if (hb_parni(2) > 0)
    MinMax->ptMinTrackSize.x = hb_parni(2);
  if (hb_parni(3) > 0)
    MinMax->ptMinTrackSize.y = hb_parni(3);
  if (hb_parni(4) > 0)
    MinMax->ptMaxTrackSize.x = hb_parni(4);
  if (hb_parni(5) > 0)
    MinMax->ptMaxTrackSize.y = hb_parni(5);
}


        //GetRectArray(pRect, aRect)
HB_FUNC ( GETRECTARRAY )
{
  RECT *rc = (RECT *) HMG_parnl(1);

  hb_storvni(rc->left,   2, 1);
  hb_storvni(rc->top,    2, 2);
  hb_storvni(rc->right,  2, 3);
  hb_storvni(rc->bottom, 2, 4);
}


        //SetRectArray(pRect, aRect)
HB_FUNC ( SETRECTARRAY )
{
  RECT *rc = (RECT *) HMG_parnl(1);

  rc->left   = hb_parvni(2, 1);
  rc->top    = hb_parvni(2, 2);
  rc->right  = hb_parvni(2, 3);
  rc->bottom = hb_parvni(2, 4);
}

#pragma ENDDUMP
